<?php
/******************************************************************************
Description : vpTransformationen 1.0 - functions
Author      : Ingo van Peeren
Urls        : http://www.van-peeren.de
Create date : 2005-01-02
Modified    : 2006-12-13
License     : GNU GPL (see folder "docs/license.txt")
******************************************/

cInclude("classes", "class.transformations.php");

if (!function_exists('html_entity_decode')) {
	function html_entity_decode($given_html, $quote_style = ENT_QUOTES)
	{
		$trans_table = array_flip(get_html_translation_table(HTML_SPECIALCHARS, $quote_style));
		$trans_table['&#39;'] = "'";
		return (strtr($given_html, $trans_table));
	} 
}  

function trans_counter($string, &$count) {

  $count++;
  return " " . $string . $count . " ";

}

function tags_counter($match) {
  global $htmltrans, $htmlcount;
  
  $htmlcount++;
  $htmltrans[' VPTHTML'.$htmlcount.' '] = $match[0]; 
  return ' VPTHTML'.$htmlcount.' ';
}

function Transform($text)
{
	global $cfg, $lang, $debug, $htmltrans;

	static $search = array();
	static $replace = array();
	static $gotTransformations = 0; 
	
	$vpTransformationen = new vpTransformationCollection;

	$acronym_behavior      = getEffectiveSetting("vptrans", "acronym-behavior", "all");
	$link_behavior         = getEffectiveSetting("vptrans", "link-behavior", "all");
	$explanation_behavior  = getEffectiveSetting("vptrans", "explanation-behavior", "all");
	$exactmatch            = getEffectiveSetting("vptrans", "exact_match", false);
	$externallinkclass     = getEffectiveSetting("vptrans", "externallink-class", "externallink");
	$abbrclass             = getEffectiveSetting("vptrans", "abbr-class", "abbr");
	$languageclass         = getEffectiveSetting("vptrans", "language-class", "language");
	$explanationclass      = getEffectiveSetting("vptrans", "explanation-class", "explanation");
	
	$ignorearr   = array();
	$hrefarr     = array();
	$acronymarr  = array();
	$abbrarr     = array();
	$htmlarr     = array();
	$linkarr     = array();
	
	$htmltrans   = array();

	preg_match_all("=(.*)(<body[^>]*>)(.*)(</body>)(.*)=si", $text, $help);
	if ($help[3][0]) {
		$text = $help[3][0]; 	
		
		// Step -1 - remove all parts to ignore
		$ignorecount = preg_match_all("'<!-- VPTRANS_IGNORE START -->(.*?)<!-- VPTRANS_IGNORE ENDE -->'si", $text, $ignore);
		for ($i = 0; $i < $ignorecount; $i++) {
			$ignorearr[$i] = ' VPTIGNORE'.$i.' ';
		} 
		
		$ignorecount = -1;

		$text = preg_replace("'<!-- VPTRANS_IGNORE START -->(.*?)<!-- VPTRANS_IGNORE ENDE -->'sie", "trans_counter('VPTIGNORE', \$ignorecount)", $text);		
		
		// Step 0 - remove all links
		$hrefcount = preg_match_all("'(<a\s{1}[^>]*>)(.*?)(<[\\\]?/a>)'si", $text, $href);
	
		for ($i = 0; $i < $hrefcount; $i++) {
			$hrefarr[$i] = ' VPTHREF'.$i.' ';
			$hreftrans[' VPTHREF'.$i.' '] = $href[0][$i];
		} 
    
		$hrefcount = -1;

		$text = preg_replace("'(<a\s{1}[^>]*>)(.*?)(<[\\\]?/a>)'sie", "trans_counter('VPTHREF', \$hrefcount)", $text);    	
		
		// Step 0.1 - remove all acronym tags, we do not want to change them!!
		$acronymcount = preg_match_all("'<acronym\s{1}[^>]*>.*?<[\\\]?/acronym>'si", $text, $acronym);
		for ($i = 0; $i < $acronymcount; $i++) {
			$text = preg_replace('/(' . preg_quote($acronym[0][$i], '/') . ')/', " VPTACRONYM{$i} ", $text, 1);
			$acronymarr[$i] = ' VPTACRONYM'.$i.' ';
		} 	
		
		// Step 0.2 - remove all abbr tags, we do not want to change them!!
		$abbrcount = preg_match_all("'<abbr\s{1}[^>]*>.*?<[\\\]?/abbr>'si", $text, $abbr);
		for ($i = 0; $i < $abbrcount; $i++) {
			$text = preg_replace('/(' . preg_quote($abbr[0][$i], '/') . ')/', " VPTABBR{$i} ", $text, 1);
			$abbrarr[$i] = ' VPTABBR'.$i.' ';
		} 
  
		// Step 0.3 - remove all textarea tags, we do not want to change them!!
		$textareacount = preg_match_all("'<textarea[^>]*>.*?<[\\\]?/textarea>'si", $text, $textarea);
		for ($i = 0; $i < $textareacount; $i++) {
			$text = preg_replace('/(' . preg_quote($textarea[0][$i], '/') . ')/', " VPTTEXTAREA{$i} ", $text, 1);
			$textareaarr[$i] = ' VPTTEXTAREA'.$i.' ';
		} 
  
		// Step 0.4 - remove all option tags, we do not want to change them!!
		$optioncount = preg_match_all("'<option[^>]*>.*?<[\\\]?/option>'si", $text, $option);
		for ($i = 0; $i < $optioncount; $i++) {
			$text = preg_replace('/(' . preg_quote($option[0][$i], '/') . ')/', " VPTOPTION{$i} ", $text, 1);
			$optionarr[$i] = ' VPTOPTION'.$i.' ';
		} 
		
		$htmlcount = -1;
		$text = preg_replace_callback("/<(?:[^\"\']+?|.+?(?:\"|\').*?(?:\"|\')?.*?)*?>/si", "tags_counter", $text);	
		
		// Step 3 - move all urls starting with http:// etc. out of the way
		$linkcount = preg_match_all("/(http|https|ftp|ftps|news)\:\/\/([a-zA-Z0-9\-\._]+[\.]{1}[a-zA-Z]{2,6})(\/[a-zA-Z0-9\-\.\?\,\'\/\\\+&%\$#_=~]+)?/siU", $text, $links);
		for($i = 0; $i < $linkcount; $i++) {
			$linkarr[$i] = ' VPTURL'.$i.' ';
		} 
		
		$linkcount = -1;

		$text = preg_replace("/(http|https|ftp|ftps|news)\:\/\/([a-zA-Z0-9\-\._]+[\.]{1}[a-zA-Z]{2,6})(\/[a-zA-Z0-9\-\.\?\,\'\/\\\+&%\$#_=~]+)?/siUe", "trans_counter('VPTURL', \$linkcount)", $text);

		if (! $gotTransformations) {
			$gotTransformations = 1;

      $vpTransformationen->loadByLangAndType($lang);
       				
			// Create search/replace array from abbreviations/links information
			$i = 0;
			while ($vpTransformation = $vpTransformationen->next()) {
				$i++;
				
				$tmp['short'] = $vpTransformation->get("short");
				$tmp['textlong'] = $vpTransformation->get("textlong");
				$tmp['title'] = $vpTransformation->get("title");
				$tmp['type'] = $vpTransformation->get("type");
				
				$tmp['textlong'] = preg_replace('/(\b)/', '\\1VPTTEMP', $tmp['textlong']);
				$tmp['title'] = preg_replace('/(\b)/', '\\1VPTTEMP', $tmp['title']);
				
				switch ($tmp['type']) {
				case 0:
					// 0 = Abbreviation
  				if ($tmp['textlong'] != "") $normtitle = ' title="' . htmlspecialchars($tmp['textlong']) . '"';
				  else $normtitle = "";
				  if ($tmp['title']) $xhtmllang = ' xml:lang="' . $tmp['title'] . '"';
				  else $xhtmllang = ""; 				
					
					$shelp = '(?<![\w@])(' . preg_quote($tmp['short'], '~') . '|' . preg_quote(htmlentities($tmp['short']), '~') . ')(?![\w@])(?!\w)';
					if ($exactmatch) {
						$abbr_search[] = "~" . $shelp . "~";
						$abbr_lsearch[] = "~(" . $shelp . "(?![^<]*?>))~s";
					} else {
						$abbr_search[] = "~" . $shelp . "~i";
						$abbr_lsearch[] = "~(" . $shelp . "(?![^<]*?>))~si";
					} 
					$abbr_replace[] = "<abbr" . $xhtmllang . $normtitle . "><span" . $normtitle . " class=\"" . $abbrclass . "\">\\1</span></abbr>";
				break;
				case 1:
					// 1 = Acronym
					if ($tmp['textlong'] != "") $normtitle = ' title="' . htmlspecialchars($tmp['textlong']) . '"';
          else $normtitle = "";
          if ($tmp['title']) $xhtmllang = ' xml:lang="' . $tmp['title'] . '"';
          else $xhtmllang = ""; 					
					
					$teststring = str_replace("VPTTEMP", "", $tmp['textlong']);
					$testtext = html_entity_decode($text);
					if (($acronym_behavior == "efa") && (!strpos($testtext, $teststring) === false) && (strpos($testtext, $teststring) < strpos($testtext, $tmp['short']))) {
					}
          else {
						$shelp = '(?<![\w@])(' . preg_quote($tmp['short'], '~') . '|' . preg_quote(htmlentities($tmp['short']), '~') . ')(?![\w@])(?!\w)';
						if ($exactmatch) {
							$acronym_search[] = "~" . $shelp . "~U";
							$acronym_lsearch[] = "~(" . $shelp . "(?![^<]*?>))~s";
						} else {
							$acronym_search[] = "~" . $shelp . "~iU";
              $acronym_lsearch[] = "~(" . $shelp . "(?![^<]*?>))~si";
						} 
						$acronym_replace[] = "<acronym" . $xhtmllang . $normtitle . ">\\1</acronym>";
					} 
				break;
				case 2:
					// 2 = Link
					// if short beginns with a single ' we need another regexp to not check for \w
					// this enables autolinks for german apostrophs
					
					$extclass = (preg_match("/(^http:\/\/)/", $tmp['textlong']) == 1) ? "class=\"$externallinkclass\"" : "";
					if ($tmp['title'] != "") $linktitle = ' title="' . htmlspecialchars($tmp['title']) . '"';
				  else $linktitle = "";
					
					if ($tmp['short'][0] == '\'') {
						$shelp = '(?<![@])(' . preg_quote($tmp['short'], '/') . '|' . preg_quote(htmlentities($tmp['short']), '/') . ')(?![\w@])(?!\w)';
					} else {
						$shelp = '(?<![\w@])(' . preg_quote($tmp['short'], '/') . '|' . preg_quote(htmlentities($tmp['short']), '/') . ')(?![\w@])(?!\w)';
					} 
					if ($exactmatch) $link_search[] = "/" . $shelp . "/U";
					else $link_search[] = "/" . $shelp . "/iU";
					$link_replace[] = "<a href=\"" . htmlspecialchars($tmp['textlong']) . "\"" . $linktitle . " " . $extclass . ">\\1</a>";
				break;
				case 3:
					// 3 = Language
					
				  if ($tmp['textlong'] != "") $normtitle = ' title="' . htmlspecialchars($tmp['textlong']) . '"';
				  else $normtitle = "";
				  if ($tmp['title']) $xhtmllang = ' xml:lang="' . $tmp['title'] . '"';
				  else $xhtmllang = ""; 					
					
					$shelp = '(?<![\w@])(' . preg_quote($tmp['short'], '/') . '|' . preg_quote(htmlentities($tmp['short']), '/') . ')(?![\w@])(?!\w)';
					if ($exactmatch) {
						$language_search[] = "/" . $shelp . "/";
						$language_lsearch[] = "/(" . $shelp . "(?![^<]*?>))/s";
					} else {
						$language_search[] = "/" . $shelp . "/i";
						$language_lsearch[] = "/(" . $shelp . "(?![^<]*?>))/si";
					} 
					$language_replace[] = "<span" . $xhtmllang . $normtitle . " class=\"" . $languageclass . "\">\\1</span>";
				break;
				case 5:
					// 5 = Glossar
					// if short beginns with a single ' we need another regexp to not check for \w
					// this enables autolinks for german apostrophs
					
					$extclass = (preg_match("/(^http:\/\/)/", $tmp['textlong']) == 1) ? "class=\"$externallinkclass\"" : "";
					if ($tmp['title'] != "") $linktitle = ' title="' . htmlspecialchars($tmp['title']) . '"';
				  else $linktitle = "";
					
					if ($tmp['short'][0] == '\'') {
						$shelp = '(?<![@])(' . preg_quote($tmp['short'], '/') . '|' . preg_quote(htmlentities($tmp['short']), '/') . ')(?![\w@])(?!\w)';
					} else {
						$shelp = '(?<![\w@])(' . preg_quote($tmp['short'], '/') . '|' . preg_quote(htmlentities($tmp['short']), '/') . ')(?![\w@])(?!\w)';
					} 
					if ($exactmatch) $link_search[] = "/" . $shelp . "/U";
					else $link_search[] = "/" . $shelp . "/iU";
					$link_replace[] = "<a href=\"" . htmlspecialchars($tmp['textlong']) . "\"" . $linktitle . " class=\"glosarlink\">\\1</a>";
				break;
				case 6:
					// 6 = Platzhalter
					
				  if ($tmp['textlong'] != "") $normtitle = ' ' . htmlspecialchars($tmp['textlong']) . '';
				  else $normtitle = "";				
					
					$shelp = '(?<![\w@])(' . preg_quote($tmp['short'], '/') . '|' . preg_quote(htmlentities($tmp['short']), '/') . ')(?![\w@])(?!\w)';
					
					if ($exactmatch) {
						$platzhalter_search[] = "/" . $shelp . "/";
						$platzhalter_lsearch[] = "/(" . $shelp . "(?![^<]*?>))/s";
					} else {
						$platzhalter_search[] = "/" . $shelp . "/i";
						$platzhalter_lsearch[] = "/(" . $shelp . "(?![^<]*?>))/si";
					} 
					// $platzhalter_replace[] = "<span" . $normtitle . " class=\"" . $explanationclass . "\">\\1</span>";
					$normtitle = html_entity_decode($normtitle);
					$normtitle = str_replace('\"', '"', $normtitle);
					$normtitle = str_replace('\"', '"', $normtitle);
					$normtitle = str_replace('\\\"', '"', $normtitle);
					$normtitle = str_replace('\\\\\\\"', '"', $normtitle);
					$normtitle = str_replace('\\\\\\\\\\\\\\\"', '"', $normtitle);
					$platzhalter_replace[] = "" . $normtitle . "";
				break;	
				case 4:
					// 4 = Explanation
					
				  if ($tmp['textlong'] != "") $normtitle = ' title="' . htmlspecialchars($tmp['textlong']) . '"';
				  else $normtitle = "";				
					
					$shelp = '(?<![\w@])(' . preg_quote($tmp['short'], '/') . '|' . preg_quote(htmlentities($tmp['short']), '/') . ')(?![\w@])(?!\w)';
					if ($exactmatch) {
						$explanation_search[] = "/" . $shelp . "/";
						$explanation_lsearch[] = "/(" . $shelp . "(?![^<]*?>))/s";
					} else {
						$explanation_search[] = "/" . $shelp . "/i";
						$explanation_lsearch[] = "/(" . $shelp . "(?![^<]*?>))/si";
					} 
					$explanation_replace[] = "<span" . $normtitle . " class=\"" . $explanationclass . "\">\\1</span>";
				} 
			} 
		} 
		// Step 6 - the main replacements
		$linktexts = $href[2];

		// LINK
		if ($link_behavior == "first") {
			if (is_array($link_search)) $text = preg_replace($link_search, $link_replace, $text, 1);
		} else {
			if (is_array($link_search)) $text = preg_replace($link_search, $link_replace, $text);
		}
   
		// ACRONYM
		if ($acronym_behavior == "first") {
			if (is_array($acronym_search)) {
				$text = preg_replace($acronym_search, $acronym_replace, $text, 1);
				if (is_array($linktexts)) $linktexts = preg_replace($acronym_lsearch, $acronym_replace, $linktexts, 1);
			} 
		} else {
			if (is_array($acronym_search)) {
				$text = preg_replace($acronym_search, $acronym_replace, $text);
				if (is_array($linktexts)) $linktexts = preg_replace($acronym_lsearch, $acronym_replace, $linktexts);
			} 
		} 
		
		// LANGUAGE
		if (is_array($language_search)) {
			$text = preg_replace($language_search, $language_replace, $text);
			if (is_array($linktexts)) $linktexts = preg_replace($language_lsearch, $language_replace, $linktexts);
		} 
		
		// ABBR
		if (is_array($abbr_search)) {
			$text = preg_replace($abbr_search, $abbr_replace, $text);
			if (is_array($linktexts)) $linktexts = preg_replace($abbr_lsearch, $abbr_replace, $linktexts);
		} 	
		
		// Platzhalter
		if (is_array($platzhalter_search)) {
				$text = preg_replace($platzhalter_search, $platzhalter_replace, $text);
				if (is_array($linktexts)) $linktexts = preg_replace($platzhalter_lsearch, $platzhalter_lreplace, $linktexts);
		} 		
		
		// EXPLANATION
		if ($explanation_behavior == "first") {
			if (is_array($explanation_search)) {
				$text = preg_replace($explanation_search, $explanation_replace, $text, 1);
				if (is_array($linktexts)) $linktexts = preg_replace($explanation_lsearch, $explanation_lreplace, $linktexts, 1);
			} 
		} else {
			if (is_array($explanation_search)) {
				$text = preg_replace($explanation_search, $explanation_replace, $text);
				if (is_array($linktexts)) $linktexts = preg_replace($explanation_lsearch, $explanation_lreplace, $linktexts);
			} 
		} 	

		foreach ($href[0] as $key=>$value) {
		  $href[0][$key] = $href[1][$key].$linktexts[$key].$href[3][$key];
		}

		// Step 7 - replace the spaces we munged in preparation of step 6
		$href[0] = str_replace('VPTTEMP', '', $href[0]);
		$text = str_replace('VPTTEMP', '', $text); 
		// Step 8-12 - replace the tags that we removed before
		
		$text = str_replace($linkarr, $links[0], $text);
		
		$text = strtr($text, $htmltrans);		
		
		$text = str_replace($optionarr, $option[0], $text);
  
		$text = str_replace($textareaarr, $textarea[0], $text);	
		
		for ($i = 0; $i < $hrefcount; $i++) {
			if ((preg_match("/http:/i", $href[0][$i])) && (!preg_match("/" . $_SERVER["HTTP_HOST"] . "/i", $href[0][$i]))) {
				if (preg_match("/class=[\"\']?" . $externallinkclass . "[\"\'\s]/i", $href[0][$i])) {
				} 
        elseif (preg_match("/class=[\"\']?/i", $href[0][$i])) {
					$href[0][$i] = preg_replace("/class=(\"|\')?/i", "class=\\1" . $externallinkclass . " ", $href[0][$i], 1);
				} 
        else $href[0][$i] = preg_replace("/<a[\s]/i", "<a class=\"$externallinkclass\" ", $href[0][$i], 1);
			} 
		} 
		$text = str_replace($hrefarr, $href[0], $text);	
		
		$text = str_replace($abbrarr, $abbr[0], $text);	
		
		$text = str_replace($acronymarr, $acronym[0], $text);	
		
		$text = str_replace($ignorearr, $ignore[0], $text);
		
		$transtext = $help[1][0] . $help[2][0] . $text . $help[4][0] . $help[5][0];
	} 
  else $transtext = $text;

	return $transtext;
} 

?>